import re
from pyspark.sql import DataFrame

def to_snake_case(col_str: str) -> str:
    """
    Cases Supported for Conversion to Snake Case
        1. CamelCase (lower camel case):
        • Example: myVariableName → my_variable_name
        2. PascalCase (upper camel case):
        • Example: MyVariableName → my_variable_name
        3. kebab-case (convert - to _):
        • Example: my-variable-name → my_variable_name
        4. Titles or Sentences with Spaces:
        • Example: My Variable Name → my_variable_name
        5. Dot-separated case (convert . to _):
        • Example: my.variable.name → my_variable_name
        6. Existing Snake Case (does nothing):
        • Example: my_variable_name → my_variable_name
        7. Case with Numbers:
        • Handles numbers without breaking the sequence.
        • Example: myVariable2Name → my_variable2_name
        8. MixedCase (mix of CamelCase and others):
        • Example: My_variable.2 Name → my_variable_2_name

    Args: 
        col_str(str): Column Name
    Returns: 
        snake_str(str): Snake Case Column Name
    """
    # Pre-processing for special cases
    # Handle 'ID' at the end
    if col_str.endswith('ID'):
        col_str = col_str[:-2] + 'Id'
    
    # Replace other common ID cases in the middle of the string
    col_str = re.sub(r'ID([A-Z])', r'Id\1', col_str)
    
    # Basic cleaning
    col_str = col_str.strip().replace('-', '_').replace('.', '_').replace(' ', '')
    
    # Convert to snake_case
    snake_str = re.sub(r'(?<!^)(?=[A-Z])', '_', col_str).lower()
    
    # Post-processing to clean special cases
    snake_str = snake_str.replace('_i_d', '_id')
    snake_str = snake_str.replace('e_bis', 'ebis')
    
    return snake_str

def rename_columns_to_snake_case(df: DataFrame) -> DataFrame:
    """
    Renames the columns of a Spark DataFrame to snake_case.

    Args:
        df (DataFrame): The input Spark DataFrame.

    Returns:
        DataFrame: A new DataFrame with column names in snake_case.
    """
    snake_case_columns = [to_snake_case(col) for col in df.columns]
    renamed_df = df.toDF(*snake_case_columns)
    return renamed_df
